/* MEX function: readFromArduino
 * Description:  open, close usb port, serial reading data bytes from arduino
 * Usage:
 *  fd=readFromArduino('port_name') - open port and return file descriptor
 *  readFromArduino(fd) - close file descriptor
 *  data=readFromArduino(fd,number_of_bytes) - read <number_of_bytes> data bytes from file descriptor
 */


#include "mex.h"

#include <fcntl.h>
#include <unistd.h>
/* The computational routine */
int serialport_read(int fd, double* buf, int buf_max, int timeout)
{
    int b[1];
    int i=0;
    do { 
        int n = read(fd, b, 1);
        if( n==-1) return -1;
        if( n==0 ) {
            usleep( 1 * 1000 );
            timeout--;
            if( timeout==0 ) return -2;
            continue;
        }
        buf[i] = (double) b[0]; 
        i++;
    } while(i < buf_max && timeout>0 );

    buf[i] = 0;
    return 0;
}

void readFromArduino(double *z, int columns, int fd) {
	int buf_max=columns;
	int timeout=5000;
	serialport_read(fd, z, buf_max, timeout);
}

/* The gateway function */
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    char errmsg[] = "usage:\n fd=readFromArduino('port_name') - open port\n data=readFromArduino(fd,number_of_bytes) - read data\n readFromArduino(fd) - close port";
    
    /* checking arguments */
	if(nrhs == 1 && nlhs == 1) { /* OPEN */
		/* make sure the argument is text */
        if( !mxIsChar(prhs[0])) {
            mexErrMsgIdAndTxt("MyToolbox:readFromArduino", errmsg);
        }
        int n = mxGetNumberOfElements(prhs[0]);
        char port_name[n+1];
        mxGetString(prhs[0],port_name,(n+1)*sizeof(char)); /* convert matlab string to C string */
        
        /* create the output matrix */
        plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
        /* get a pointer to the real data in the output matrix */
        double *fd_ptr = mxGetPr(plhs[0]);
        
        int fd = open(port_name, O_RDWR | O_NONBLOCK ); /* open arduino port */
        fd_ptr[0] = fd;
	} else if(nrhs == 2 && nlhs == 1) { /* READ DATA */
        /* make sure the arguments are scalars */
        if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||	mxGetNumberOfElements(prhs[0]) != 1 ) {
            mexErrMsgIdAndTxt("MyToolbox:readFromArduino", errmsg);
        }
        if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||	mxGetNumberOfElements(prhs[1]) != 1 ) {
            mexErrMsgIdAndTxt("MyToolbox:readFromArduino", errmsg);
        }
        
        int fd = (int) mxGetScalar(prhs[0]);              /* file descriptor */
        int columns = (int) mxGetScalar(prhs[1]);	      /* number of columns */
        if (columns <= 0) {
            mexErrMsgIdAndTxt("MyToolbox:readFromArduino", "second input must be positive");
        }
        
        /* create the output matrix */
        plhs[0] = mxCreateDoubleMatrix(1,columns,mxREAL);
        /* get a pointer to the real data in the output matrix */
        double *outMatrix = mxGetPr(plhs[0]);
	
        /* call the computational routine */
        readFromArduino(outMatrix,columns,fd);
    } else if(nrhs == 1 && nlhs == 0) { /* CLOSE */
        /* make sure the argument is scalar */
        if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||	mxGetNumberOfElements(prhs[0]) != 1 ) {
            mexErrMsgIdAndTxt("MyToolbox:readFromArduino", errmsg);
        }
        
        int fd = (int) mxGetScalar(prhs[0]);              /* file descriptor */
        close(fd);                                        /* closing arduino port */
    } else {
        mexErrMsgIdAndTxt("MyToolbox:readFromArduino",errmsg);
    }
	
	
}
